DeathClock = CreateFrame("Frame")
DeathClock.logAge = 10
DeathClock.align = "CENTER"
DeathClock.DB = {}
DeathClock.Frames = {}
DeathClock.debug = {}

function DeathClock:OnEvent(event, ...)
	-- Event Handler
	self[event](self, event, ...)
end

function DeathClock:ADDON_LOADED(event, addonName)
	if (addonName == "DeathClock") then
		--Load Saved Variables
	end
end

function DeathClock:PLAYER_LOGIN()
	if IsAddOnLoaded("ElvUI") then
		DeathClock.CreateOrUpdateFrame("player", ElvUF_Player.Health)
		DeathClock.CreateOrUpdateFrame("pet", ElvUF_Pet.Health)
		DeathClock.CreateOrUpdateFrame("pettarget", ElvUF_PetTarget.Health)
		DeathClock.CreateOrUpdateFrame("target", ElvUF_Target.Health)
		DeathClock.CreateOrUpdateFrame("targettarget", ElvUF_TargetTarget.Health)
		DeathClock.CreateOrUpdateFrame("targettargettarget", ElvUF_TargetTargetTarget.Health)
		DeathClock.CreateOrUpdateFrame("focus", ElvUF_Focus.Health)
		DeathClock.CreateOrUpdateFrame("focustarget", ElvUF_FocusTarget.Health)
	else
		DeathClock.CreateOrUpdateFrame("player", PlayerFrame.PlayerFrameContent.PlayerFrameContentMain.HealthBarArea.HealthBar, true)
		DeathClock.CreateOrUpdateFrame("pet", PetFrameHealthBar)
		DeathClock.CreateOrUpdateFrame("target", TargetFrame.TargetFrameContent.TargetFrameContentMain.HealthBar, true)
		DeathClock.CreateOrUpdateFrame("targettarget", TargetFrameToT.HealthBar)
		DeathClock.CreateOrUpdateFrame("focus", FocusFrame.TargetFrameContent.TargetFrameContentMain.HealthBar, true)
		DeathClock.CreateOrUpdateFrame("focustarget", FocusFrameToT.HealthBar)	
	end
end

function DeathClock:NAME_PLATE_UNIT_ADDED(event, unitID)
	for i, namePlate in ipairs(C_NamePlate.GetNamePlates()) do
		-- Blizzard = namePlate.UnitFrame.healthBar
		-- Plater = namePlate.unitFrame.healthBar
		-- ElvUI = namePlate.UnitFrame.Health
		--print(namePlate.namePlateUnitToken)
		unitFrame = namePlate.unitFrame or namePlate.UnitFrame
		healthBar = unitFrame.healthBar or unitFrame.Health
		DeathClock.CreateOrUpdateFrame(namePlate.namePlateUnitToken, healthBar, false, DeathClock.align)
	end
end

function DeathClock:NAME_PLATE_UNIT_REMOVED(event, unitID)
	DeathClock:NAME_PLATE_UNIT_ADDED(event, unitID)
end

function DeathClock.Debug()
	local plates = C_NamePlate.GetNamePlates()
	return plates[1].unitFrame
end

function DeathClock.CreateOrUpdateFrame(unitID, attachFrame, hideOnMouseOver, JustifyH)
	if (not DeathClock.Frames[unitID]) then
		DeathClock.Frames[unitID] = DeathClockFrame:new(unitID, attachFrame, hideOnMouseOver, JustifyH)
	else
		DeathClock.Frames[unitID]:Update(unitID, attachFrame)
	end
end

function tprint (tbl, indent)
  if not indent then indent = 0 end
  local toprint = string.rep(" ", indent) .. "{\r\n"
  indent = indent + 2 
  for k, v in pairs(tbl) do
    toprint = toprint .. string.rep(" ", indent)
    if (type(k) == "number") then
      toprint = toprint .. "[" .. k .. "] = "
    elseif (type(k) == "string") then
      toprint = toprint  .. k ..  "= "   
    end
    if (type(v) == "number") then
      toprint = toprint .. v .. ",\r\n"
    elseif (type(v) == "string") then
      toprint = toprint .. "\"" .. v .. "\",\r\n"
    elseif (type(v) == "table") then
      toprint = toprint .. tprint(v, indent + 2) .. ",\r\n"
    else
      toprint = toprint .. "\"" .. tostring(v) .. "\",\r\n"
    end
  end
  toprint = toprint .. string.rep(" ", indent-2) .. "}"
  return toprint
end


function DeathClock:COMBAT_LOG_EVENT_UNFILTERED()
	-- Get combat log data
	
	-- event = Event Name
	-- arg1  = Timestamp or Addon Name
	-- arg2  = Damage Type
	-- arg8  = Target GUID
	-- arg12 = Damage Amount (SWING)			extraArg1
	-- arg13 = Damage Amount (ENVIRONMENTAL)	extraArg2
	-- arg15 = Damage Amount (all other types)	extraArg4
	--local timestamp, damageType, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags, extraArg1, extraArg2, extraArg3, extraArg4, extraArg5, extraArg6, extraArg7, extraArg8, extraArg9, extraArg10 = CombatLogGetCurrentEventInfo();
	local timestamp, damageType, _, _, _, _, _, destGUID, _, _, _, extraArg1, extraArg2, _, extraArg4 = CombatLogGetCurrentEventInfo()
	if (damageType == "SWING_DAMAGE") then
		DeathClock:logDamage(timestamp, destGUID, extraArg1)
	elseif (damageType == "SPELL_DAMAGE") then
		DeathClock:logDamage(timestamp, destGUID, extraArg4)
	elseif (damageType == "RANGE_DAMAGE") then
		DeathClock:logDamage(timestamp, destGUID, extraArg4)
	elseif (damageType == "SPELL_PERIODIC_DAMAGE") then
		DeathClock:logDamage(timestamp, destGUID, extraArg4)
	elseif (damageType == "SPELL_BUILDING_DAMAGE") then
		DeathClock:logDamage(timestamp, destGUID, extraArg4)
	elseif (damageType == "DAMAGE_SHIELD") then
		DeathClock:logDamage(timestamp, destGUID, extraArg4)
	elseif (damageType == "ENVIRONMENTAL_DAMAGE") then
		DeathClock:logDamage(timestamp, destGUID, extraArg2)
	end
end

function DeathClock:logDamage(timestamp, guID, damage)
	-- Add damage logs to database
	--print(timestamp.." | "..guID.." | "..damage);
	local record = {timestamp, guID, damage};
	tinsert(DeathClock.DB, 1, record);
end

function round(x)
	if x % 1 >= 0.5 then
		return ceil(x)
	else
		return floor(x)
	end
end

function DeathClock:getDamageOnUnit(unitGUID)
	-- tableformat = {time, guid, damage}
	local totalDamage = 0;
	for index, record in ipairs(DeathClock.DB) do
		if (time() - record[1]) > DeathClock.logAge then
			-- Delete Expired Data
			tremove(DeathClock.DB, index);
		elseif record[2] == unitGUID then
			totalDamage = totalDamage + record[3];
		end	
	end
	local dps = round(totalDamage / DeathClock.logAge);
	return dps;
end

function DeathClock:getETD(health, dps)
	local etd;
	if (dps > 0) then
		etd = health / dps;
	else
		etd = -1;
	end
	return etd;
end

function DeathClock:formatTime(seconds)
	if (seconds < 1) then -- Less than 1 second
		return "";
	elseif (seconds < 120) then -- Less than 120 seconds
		return round(seconds) .. "s";
	elseif ((seconds / 60) < 60) then -- Less than 60 minutes
		return round(seconds / 60) .. "m";
	elseif ((seconds / 60 / 60) < 24) then -- Less than 24 hours
		return round(seconds / 60 / 60) .. "h";
	elseif ((seconds / 60 / 60 / 24) < 365) then -- Less than 365 days
		return round(seconds / 60 / 60 / 24) .. "d";
	elseif ((seconds / 60 / 60 / 24 / 365) < 99) then -- Less than 99 years
		return round(seconds / 60 / 60 / 24) .. "y";
	else -- Greater than 99 years
		return("∞");
	end
end

DeathClock:RegisterEvent("ADDON_LOADED")
DeathClock:RegisterEvent("PLAYER_LOGIN")
DeathClock:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
DeathClock:RegisterEvent("NAME_PLATE_UNIT_ADDED")
DeathClock:RegisterEvent("NAME_PLATE_UNIT_REMOVED")
DeathClock:SetScript("OnEvent", DeathClock.OnEvent)


DeathClockFrame = {}
function DeathClockFrame:new(unitID, attachFrame, hideOnMouseOver, JustifyH)
	self = {}
	
	-- Member Variables:
	self.unitID = unitID
	self.attachFrame = attachFrame
	self.hideOnMouseOver = hideOnMouseOver
	self.JustifyH = JustifyH or "CENTER"
	self.test = "NA"
	self.lastUpdate = 0
	
	-- Member Functions:
	function self.OnUpdate()
		if time() > self.lastUpdate then
			self:Refresh()
		end
	end
	
	function self.OnEvent(_, event, arg1)
		if (event == "PLAYER_TARGET_CHANGED") then
			self:Refresh()
		elseif (event == "NAME_PLATE_UNIT_ADDED") then
			self:Refresh()
		end
	end
	
	function self.Refresh()
		if self.hideOnMouseOver and MouseIsOver(self.overlay) then
			self.overlay:SetText("")
		else
			--if UnitGUID(self.unitID) == nil then print("nil") end
			dps = DeathClock:getDamageOnUnit(UnitGUID(self.unitID))
			etd = DeathClock:getETD(UnitHealth(self.unitID), dps)
			self.overlay:SetText(DeathClock:formatTime(etd))
			self.lastUpdate = time()
		end
	end
	
	function self.Update(_, unitID, attachFrame)
		--print(unitID)
		self.unitID = unitID
		self.attachFrame = attachFrame
		self.overlay:ClearAllPoints()
		self.overlay:SetPoint("TOPLEFT", self.attachFrame, "TOPLEFT", 0, 0)
		self.overlay:SetPoint("BOTTOMRIGHT", self.attachFrame, "BOTTOMRIGHT", 0, 0)
		
	end
	
	self.frame = CreateFrame("Frame")
	self.frame:RegisterEvent("PLAYER_TARGET_CHANGED")
	self.frame:RegisterEvent("NAME_PLATE_UNIT_ADDED")
	self.frame:SetScript("OnEvent", self.OnEvent)
	self.frame:HookScript("OnUpdate", self.OnUpdate)
	
	self.overlay = self.frame:CreateFontString("DeathClockFont"..self.unitID, "ARTWORK", "TextStatusBarTextLarge")
	self.overlay:SetPoint("TOPLEFT", self.attachFrame, "TOPLEFT", 0, 0)
	self.overlay:SetPoint("BOTTOMRIGHT", self.attachFrame, "BOTTOMRIGHT", 0, 0)
	self.overlay:SetJustifyH(self.JustifyH)
	
	if self.hideOnMouseOver then
		self.attachFrame:HookScript("OnEnter", self.Refresh)
	end
	
	return self
end